--[[
	AutomaticUnloadForBaleLoader.lua
	UhrModder
	Author: 	Ifko[nator]
	Date: 		19.12.2018
	Version:	1.0
	
	Geändert
	Autor: 		Thunder01
	Datum: 		28.08.2020 
	Version: 	1.0
	
	Changelog:	V 1.0 @ 28.08.2020 - initial implementation in FS 19
]]

AutomaticUnloadForBaleLoader = {};
AutomaticUnloadForBaleLoader.DROP_MODE_MANUAL = 1;
AutomaticUnloadForBaleLoader.DROP_MODE_AUTOMATIC = 2;
AutomaticUnloadForBaleLoader.DROP_MODE_COLLECT = 3;
AutomaticUnloadForBaleLoader.DROP_MODE_MAX_STATE = 3;
AutomaticUnloadForBaleLoader.FORCE_DROP_BALE = false;

function AutomaticUnloadForBaleLoader.prerequisitesPresent(specializations)
	return true;
end;

function AutomaticUnloadForBaleLoader.registerEventListeners(vehicleType)
	local functionNames = {
		"onLoad",
		"onUpdate",
		"saveToXMLFile",
		"onWriteStream",
		"onReadStream",
		"onRegisterActionEvents"
	};
	
	for _, functionName in ipairs(functionNames) do
		SpecializationUtil.registerEventListener(vehicleType, functionName, AutomaticUnloadForBaleLoader);
	end;
end;

function AutomaticUnloadForBaleLoader.registerFunctions(vehicleType)
	local functionNames = {
		"setDropMode",
		"toggleBaleLoader"
	};
	
	for _, functionName in ipairs(functionNames) do
		SpecializationUtil.registerFunction(vehicleType, functionName, AutomaticUnloadForBaleLoader[functionName]);
	end;
end;

function AutomaticUnloadForBaleLoader:onLoad(savegame)
	local specAutomaticUnloadForBaleLoader = self.spec_automaticUnloadForBaleLoader;
	local specBaler = self.spec_baler;
	
	if savegame ~= nil then
		specAutomaticUnloadForBaleLoader.currentDropModeState = Utils.getNoNil(getXMLInt(savegame.xmlFile, savegame.key .. ".automaticUnloadForBaleLoader#currentDropModeState"), AutomaticUnloadForBaleLoader.DROP_MODE_MANUAL);
		
		if specBaler ~= nil then	
			specAutomaticUnloadForBaleLoader.baleLoaderIsActive = Utils.getNoNil(getXMLBool(savegame.xmlFile, savegame.key .. ".automaticUnloadForBaleLoader#baleLoaderIsActive"), true);
		end;
	else
		specAutomaticUnloadForBaleLoader.currentDropModeState = AutomaticUnloadForBaleLoader.DROP_MODE_MANUAL;
		
		if specBaler ~= nil then
			specAutomaticUnloadForBaleLoader.baleLoaderIsActive = true;
		end;
	end;
	
	self:setDropMode(specAutomaticUnloadForBaleLoader.currentDropModeState, false);
	
	if specBaler ~= nil then
		self:toggleBaleLoader(specAutomaticUnloadForBaleLoader.baleLoaderIsActive);
		
		self.doStateChange = Utils.overwrittenFunction(self.doStateChange, AutomaticUnloadForBaleLoader.doStateChange);
		self.pickupLoaderBale = Utils.overwrittenFunction(self.pickupLoaderBale, AutomaticUnloadForBaleLoader.pickupLoaderBale);
	end;
end;

function AutomaticUnloadForBaleLoader:setDropMode(currentDropModeState, noEventSend)
    local specAutomaticUnloadForBaleLoader = self.spec_automaticUnloadForBaleLoader;

	if currentDropModeState ~= specAutomaticUnloadForBaleLoader.currentDropModeState then
		specAutomaticUnloadForBaleLoader.currentDropModeState = currentDropModeState;
		
		 if not noEventSend then
			if g_server ~= nil then
				g_server:broadcastEvent(SetDropModeEvent:new(self, currentDropModeState), nil, nil, self);
			else
				g_client:getServerConnection():sendEvent(SetDropModeEvent:new(self, currentDropModeState));
			end;
		end;
	end;
end;

function AutomaticUnloadForBaleLoader:onWriteStream(streamId, connection)
	if not connection:getIsServer() then 
		local specAutomaticUnloadForBaleLoader = self.spec_automaticUnloadForBaleLoader;
		local specBaler = self.spec_baler;
		
		streamWriteUIntN(streamId, specAutomaticUnloadForBaleLoader.currentDropModeState, AutomaticUnloadForBaleLoader.DROP_MODE_MAX_STATE);

		if specBaler ~= nil then	
			streamWriteBool(streamId, specAutomaticUnloadForBaleLoader.baleLoaderIsActive);
		end;
	end;
end;

function AutomaticUnloadForBaleLoader:onReadStream(streamId, connection)
	if connection:getIsServer() then
		local specAutomaticUnloadForBaleLoader = self.spec_automaticUnloadForBaleLoader;
		local specBaler = self.spec_baler;
		
		specAutomaticUnloadForBaleLoader.currentDropModeState = streamReadUIntN(streamId, AutomaticUnloadForBaleLoader.DROP_MODE_MAX_STATE);
		
		if specBaler ~= nil then
			specAutomaticUnloadForBaleLoader.baleLoaderIsActive = streamReadBool(streamId);
		end;
	end;
end;

function AutomaticUnloadForBaleLoader:onRegisterActionEvents(isActiveForInput)
	if self.isClient then
        local specAutomaticUnloadForBaleLoader = self.spec_automaticUnloadForBaleLoader;
        
		self:clearActionEventsTable(specAutomaticUnloadForBaleLoader.actionEvents);
        
		if self:getIsActiveForInput(true) then
            local newFunctions = {
				"CHANGE_DROP_MODE_BUTTON",
				"TOGGLE_BALE_LOADER_BUTTON"
			};
			
			for _, newFunction in ipairs(newFunctions) do
				local _, actionEventId = self:addActionEvent(specAutomaticUnloadForBaleLoader.actionEvents, InputAction[newFunction], self, AutomaticUnloadForBaleLoader.actionEventChangeDropeMode, false, true, false, true, nil);
			
				g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_NORMAL);
				g_inputBinding:setActionEventTextVisibility(actionEventId, true);
				g_inputBinding:setActionEventActive(actionEventId, false);
			end;
		end;
	end;
end;

function AutomaticUnloadForBaleLoader.actionEventChangeDropeMode(self, actionName, inputValue, callbackState, isAnalog)
	local specAutomaticUnloadForBaleLoader = self.spec_automaticUnloadForBaleLoader;
	local specBaler = self.spec_baler;
	
	if actionName == "CHANGE_DROP_MODE_BUTTON" then
		if specAutomaticUnloadForBaleLoader.currentDropModeState < AutomaticUnloadForBaleLoader.DROP_MODE_MAX_STATE then
			self:setDropMode(specAutomaticUnloadForBaleLoader.currentDropModeState + 1);
		else
			self:setDropMode(AutomaticUnloadForBaleLoader.DROP_MODE_MANUAL);
		end;
	else
		if specBaler ~= nil then	
			self:toggleBaleLoader(not specAutomaticUnloadForBaleLoader.baleLoaderIsActive);
		end;
	end;
end;

function AutomaticUnloadForBaleLoader:doStateChange(superFunc, id, nearestBaleServerId)
    local specAutomaticUnloadForBaleLoader = self.spec_automaticUnloadForBaleLoader;
	local specBaleLoader = self.spec_baleLoader;
	
	if id == BaleLoader.CHANGE_LOADER_START and not specAutomaticUnloadForBaleLoader.baleLoaderIsActive then
        specBaleLoader.baleLoaderState = BaleLoader.STATE_LOADER_FINSIHED;
        
		return;
    end;
	
	superFunc(self, id, nearestBaleServerId);
end;

function AutomaticUnloadForBaleLoader:pickupLoaderBale(superFunc, bale, baleType)
    local specAutomaticUnloadForBaleLoader = self.spec_automaticUnloadForBaleLoader;
	
	if not specAutomaticUnloadForBaleLoader.baleLoaderIsActive then 
        g_server:broadcastEvent(BaleLoaderStateEvent:new(self, BaleLoader.CHANGE_GRAB_BALE, NetworkUtil.getObjectId(bale)), true, nil, self);
    else
        superFunc(self, bale, baleType);
    end;
end;

function AutomaticUnloadForBaleLoader:toggleBaleLoader(baleLoaderIsActive, noEventSend)
	local specAutomaticUnloadForBaleLoader = self.spec_automaticUnloadForBaleLoader;
	
	if baleLoaderIsActive ~= specAutomaticUnloadForBaleLoader.baleLoaderIsActive then
		if not noEventSend then
			if g_server ~= nil then
				g_server:broadcastEvent(ToggleBaleLoaderEvent:new(self, baleLoaderIsActive), nil, nil, self);
			else
				g_client:getServerConnection():sendEvent(ToggleBaleLoaderEvent:new(self, baleLoaderIsActive));
			end;
		end;
		
		specAutomaticUnloadForBaleLoader.baleLoaderIsActive = baleLoaderIsActive;
	end;
end;

function AutomaticUnloadForBaleLoader:saveToXMLFile(xmlFile, key)
	local specAutomaticUnloadForBaleLoader = self.spec_automaticUnloadForBaleLoader;
	local specBaler = self.spec_baler;
	
	setXMLInt(xmlFile, key .. "#currentDropModeState", specAutomaticUnloadForBaleLoader.currentDropModeState);
	
	if specBaler ~= nil then	
		setXMLBool(xmlFile, key .. "#balerLoaderIsActive", specAutomaticUnloadForBaleLoader.baleLoaderIsActive);
	end;
end;

function AutomaticUnloadForBaleLoader:onUpdate(dt, isActiveForInput, isSelected)
	if self:getIsActive() then
		local specAutomaticUnloadForBaleLoader = self.spec_automaticUnloadForBaleLoader;
		local specBaleLoader = self.spec_baleLoader;
		local specBaler = self.spec_baler;
		
		if specBaler == nil then
			--## disable the collect mode for single bale loaders
			AutomaticUnloadForBaleLoader.DROP_MODE_MAX_STATE = 2;
		else
			AutomaticUnloadForBaleLoader.DROP_MODE_MAX_STATE = 3;
			
			if specBaleLoader.baleToLoad == nil then
				if specBaleLoader.baleLoaderState < BaleLoader.STATE_MOVING_BALE_TO_LOADER or specBaleLoader.baleLoaderState >= BaleLoader.STATE_LOADER_LOADING_BALE then
					if specAutomaticUnloadForBaleLoader.baleLoaderIsActive and self.isServer then
						local bale = NetworkUtil.getObject(specBaleLoader.currentLoader.currentBale);
						
						if bale ~= nil and bale.loadingState == 0 and specBaleLoader.baleLoaderState == BaleLoader.STATE_LOADER_FINSIHED then
							self:playMoveToLoader(bale);
						end;
					end;
				end;
			end;
			
			local toggleBaleLoaderButton = specAutomaticUnloadForBaleLoader.actionEvents[InputAction.TOGGLE_BALE_LOADER_BUTTON];
			
			if toggleBaleLoaderButton ~= nil then
				local currentText = Vehicle.START_BALE_LOADER;
				
				if specAutomaticUnloadForBaleLoader.baleLoaderIsActive then
					currentText = Vehicle.STOP_BALE_LOADER;
				end;
				
				g_inputBinding:setActionEventActive(toggleBaleLoaderButton.actionEventId, true);
				g_inputBinding:setActionEventText(toggleBaleLoaderButton.actionEventId, currentText);
			end;
		end;
		
		local changeDropModeButton = specAutomaticUnloadForBaleLoader.actionEvents[InputAction.CHANGE_DROP_MODE_BUTTON];
		
		if changeDropModeButton ~= nil then
			local currentText = Vehicle.DROP_MODE_MANUALLY;
			
			g_inputBinding:setActionEventActive(changeDropModeButton.actionEventId, true);
			
			if specAutomaticUnloadForBaleLoader.currentDropModeState == AutomaticUnloadForBaleLoader.DROP_MODE_AUTOMATIC then
				currentText = Vehicle.DROP_MODE_AUTOMATIC;
			elseif specAutomaticUnloadForBaleLoader.currentDropModeState == AutomaticUnloadForBaleLoader.DROP_MODE_COLLECT then
				currentText = Vehicle.DROP_MODE_COLLECT;
			end;
			
			g_inputBinding:setActionEventText(changeDropModeButton.actionEventId, string.gsub(Vehicle.CURRENT_DROP_MODE, "currentDropMode", currentText));
		end;
		
		if specBaleLoader.baleLoaderState == BaleLoader.STATE_LOADER_FINSIHED and specAutomaticUnloadForBaleLoader.currentDropModeState > AutomaticUnloadForBaleLoader.DROP_MODE_MANUAL then
			local allowDropBale = false;
			
			if specAutomaticUnloadForBaleLoader.currentDropModeState == AutomaticUnloadForBaleLoader.DROP_MODE_AUTOMATIC then
				allowDropBale = true;
			elseif AutomaticUnloadForBaleLoader.FORCE_DROP_BALE then
				allowDropBale = true;
				
				AutomaticUnloadForBaleLoader.FORCE_DROP_BALE = false;
			elseif specAutomaticUnloadForBaleLoader.currentDropModeState == AutomaticUnloadForBaleLoader.DROP_MODE_COLLECT then
				if self.spec_fillUnit:getFillUnitFillLevel(1) == self.spec_fillUnit:getFillUnitCapacity(1) then
					allowDropBale = true;
					
					AutomaticUnloadForBaleLoader.FORCE_DROP_BALE = true;
				end;
			end;
			
			if self.isClient and allowDropBale then
				g_client:getServerConnection():sendEvent(BaleLoaderStateEvent:new(self, BaleLoader.CHANGE_BUTTON_EMPTY));
			end;
		end;
	end;
end;

SetDropModeEvent = {};
SetDropModeEvent_mt = Class(SetDropModeEvent, Event);
InitEventClass(SetDropModeEvent, "SetDropModeEvent");

function SetDropModeEvent:emptyNew()
    local self = Event:new(SetDropModeEvent_mt);
    self.className = "SetDropModeEvent";
    return self;
end;

function SetDropModeEvent:new(baleLoader, currentDropModeState)
    local self = SetDropModeEvent:emptyNew();
	
    self.baleLoader = baleLoader;
    self.currentDropModeState = currentDropModeState;
    
	return self;
end;

function SetDropModeEvent:writeStream(streamId, connection)
    NetworkUtil.writeNodeObject(streamId, self.baleLoader);
    streamWriteInt32(streamId, self.currentDropModeState);
end;

function SetDropModeEvent:readStream(streamId, connection)
    self.baleLoader = NetworkUtil.readNodeObject(streamId);
    self.currentDropModeState = streamReadInt32(streamId);

    self:run(connection);
end;

function SetDropModeEvent:run(connection)	
	if not connection:getIsServer() then
		g_server:broadcastEvent(SetDropModeEvent:new(self.baleLoader, self.currentDropModeState), nil, connection, self.baleLoader);
	end;
	
	if self.baleLoader ~= nil then	
		self.baleLoader:setDropMode(self.currentDropModeState, true);
	end;
end;

ToggleBaleLoaderEvent = {};
ToggleBaleLoaderEvent_mt = Class(ToggleBaleLoaderEvent, Event);

InitEventClass(ToggleBaleLoaderEvent, "ToggleBaleLoaderEvent");

function ToggleBaleLoaderEvent:emptyNew()
	local self = Event:new(ToggleBaleLoaderEvent_mt);
    
	return self;
end;

function ToggleBaleLoaderEvent:new(baleLoader, baleLoaderIsActive)
	local self = ToggleBaleLoaderEvent:emptyNew();
	
	self.baleLoader = baleLoader;
	self.baleLoaderIsActive = baleLoaderIsActive;
	
	return self;
end;

function ToggleBaleLoaderEvent:readStream(streamId, connection)
	self.baleLoader = NetworkUtil.readNodeObject(streamId);
	self.baleLoaderIsActive = streamReadBool(streamId);
    
	self:run(connection);
end;

function ToggleBaleLoaderEvent:writeStream(streamId, connection)
	NetworkUtil.writeNodeObject(streamId, self.baleLoader);
	streamWriteBool(streamId, self.baleLoaderIsActive);
end;

function ToggleBaleLoaderEvent:run(connection)
	if not connection:getIsServer() then
		g_server:broadcastEvent(ToggleBaleLoaderEvent:new(self.baleLoader, self.baleLoaderIsActive), nil, connection, self.baleLoader);
	end;
	
    if self.baleLoader ~= nil then
        self.baleLoader:toggleBaleLoader(self.baleLoaderIsActive, true);
	end;
end;